@@ -92,7 +92,7 @@ impl ObjectStoreScheme {
     /// assert_eq!(scheme, ObjectStoreScheme::Local);
     /// assert_eq!(path.as_ref(), "path/to/my/file");
     ///
-    /// let url: Url = "https://blob.core.windows.net/path/to/my/file".parse().unwrap();
+    /// let url: Url = "https://blob.core.windows.net/container/path/to/my/file".parse().unwrap();
     /// let (scheme, path) = ObjectStoreScheme::parse(&url).unwrap();
     /// assert_eq!(scheme, ObjectStoreScheme::MicrosoftAzure);
     /// assert_eq!(path.as_ref(), "path/to/my/file");
@@ -110,9 +110,8 @@ impl ObjectStoreScheme {
             ("memory", None) => (Self::Memory, url.path()),
             ("s3" | "s3a", Some(_)) => (Self::AmazonS3, url.path()),
             ("gs", Some(_)) => (Self::GoogleCloudStorage, url.path()),
-            ("az" | "adl" | "azure" | "abfs" | "abfss", Some(_)) => {
-                (Self::MicrosoftAzure, url.path())
-            }
+            ("az", Some(_)) => (Self::MicrosoftAzure, strip_bucket().unwrap_or_default()),
+            ("adl" | "azure" | "abfs" | "abfss", Some(_)) => (Self::MicrosoftAzure, url.path()),
             ("http", Some(_)) => (Self::Http, url.path()),
             ("https", Some(host)) => {
                 if host.ends_with("dfs.core.windows.net")
@@ -120,7 +119,7 @@ impl ObjectStoreScheme {
                     || host.ends_with("dfs.fabric.microsoft.com")
                     || host.ends_with("blob.fabric.microsoft.com")
                 {
-                    (Self::MicrosoftAzure, url.path())
+                    (Self::MicrosoftAzure, strip_bucket().unwrap_or_default())
                 } else if host.ends_with("amazonaws.com") {
                     match host.starts_with("s3") {
                         true => (Self::AmazonS3, strip_bucket().unwrap_or_default()),
@@ -164,9 +163,27 @@ pub fn parse_url(url: &Url) -> Result<(Box<dyn ObjectStore>, Path), super::Error
 
 /// Create an [`ObjectStore`] based on the provided `url` and options
 ///
+/// This method can be used to create an instance of one of the provided
+/// `ObjectStore` implementations based on the URL scheme (see
+/// [`ObjectStoreScheme`] for more details).
+///
+/// For example
+/// * `file:///path/to/my/file` will return a [`LocalFileSystem`] instance
+/// * `s3://bucket/path` will return an [`AmazonS3`] instance if the `aws` feature is enabled.
+///
+/// Arguments:
+/// * `url`: The URL to parse
+/// * `options`: A list of key-value pairs to pass to the [`ObjectStore`] builder.
+///   Note different object stores accept different configuration options, so
+///   the options that are read depends on the `url` value. One common pattern
+///   is to pass configuration information via process variables using
+///   [`std::env::vars`].
+///
 /// Returns
 /// - An [`ObjectStore`] of the corresponding type
 /// - The [`Path`] into the [`ObjectStore`] of the addressed resource
+///
+/// [`AmazonS3`]: https://docs.rs/object_store/0.12.0/object_store/aws/struct.AmazonS3.html
 pub fn parse_url_opts<I, K, V>(
     url: &Url,
     options: I,
@@ -202,6 +219,7 @@ where
             builder_opts!(crate::http::HttpBuilder, url, _options)
         }
         #[cfg(not(all(
+            feature = "fs",
             feature = "aws",
             feature = "azure",
             feature = "gcp",
@@ -268,10 +286,26 @@ mod tests {
                 "https://account.dfs.core.windows.net",
                 (ObjectStoreScheme::MicrosoftAzure, ""),
             ),
+            (
+                "https://account.dfs.core.windows.net/container/path",
+                (ObjectStoreScheme::MicrosoftAzure, "path"),
+            ),
             (
                 "https://account.blob.core.windows.net",
                 (ObjectStoreScheme::MicrosoftAzure, ""),
             ),
+            (
+                "https://account.blob.core.windows.net/container/path",
+                (ObjectStoreScheme::MicrosoftAzure, "path"),
+            ),
+            (
+                "az://account/container",
+                (ObjectStoreScheme::MicrosoftAzure, ""),
+            ),
+            (
+                "az://account/container/path",
+                (ObjectStoreScheme::MicrosoftAzure, "path"),
+            ),
             (
                 "gs://bucket/path",
                 (ObjectStoreScheme::GoogleCloudStorage, "path"),
@@ -286,6 +320,15 @@ mod tests {
                 "s3://bucket/foo%20bar",
                 (ObjectStoreScheme::AmazonS3, "foo bar"),
             ),
+            (
+                "s3://bucket/foo bar",
+                (ObjectStoreScheme::AmazonS3, "foo bar"),
+            ),
+            ("s3://bucket/😀", (ObjectStoreScheme::AmazonS3, "😀")),
+            (
+                "s3://bucket/%F0%9F%98%80",
+                (ObjectStoreScheme::AmazonS3, "😀"),
+            ),
             (
                 "https://foo/bar%20baz",
                 (ObjectStoreScheme::Http, "bar baz"),
@@ -308,7 +351,11 @@ mod tests {
             ),
             (
                 "https://account.dfs.fabric.microsoft.com/container",
-                (ObjectStoreScheme::MicrosoftAzure, "container"),
+                (ObjectStoreScheme::MicrosoftAzure, ""),
+            ),
+            (
+                "https://account.dfs.fabric.microsoft.com/container/path",
+                (ObjectStoreScheme::MicrosoftAzure, "path"),
             ),
             (
                 "https://account.blob.fabric.microsoft.com/",
@@ -316,7 +363,11 @@ mod tests {
             ),
             (
                 "https://account.blob.fabric.microsoft.com/container",
-                (ObjectStoreScheme::MicrosoftAzure, "container"),
+                (ObjectStoreScheme::MicrosoftAzure, ""),
+            ),
+            (
+                "https://account.blob.fabric.microsoft.com/container/path",
+                (ObjectStoreScheme::MicrosoftAzure, "path"),
             ),
         ];
 
@@ -348,7 +399,7 @@ mod tests {
     }
 
     #[tokio::test]
-    #[cfg(feature = "http")]
+    #[cfg(all(feature = "http", not(target_arch = "wasm32")))]
     async fn test_url_http() {
         use crate::client::mock_server::MockServer;
         use http::{header::USER_AGENT, Response};
